@@ -1,6 +1,7 @@
 from .random_matrix import (
     fixed_rank_random_range,
     variable_rank_random_range,
+    random_range,
     random_svd,
     random_eig,
     random_cholesky,
@@ -16,16 +17,22 @@ from .hilbert_space import (
 )
 
 
-from .operators import (
-    Operator,
-    LinearOperator,
-    DiagonalLinearOperator,
+from .nonlinear_forms import (
+    NonLinearForm,
 )
 
+
 from .linear_forms import (
     LinearForm,
 )
 
+from .nonlinear_operators import NonLinearOperator
+
+from .linear_operators import (
+    LinearOperator,
+    DiagonalLinearOperator,
+)
+
 
 from .gaussian_measure import (
     GaussianMeasure,
@@ -45,7 +52,9 @@ from .linear_solvers import (
     DirectLinearSolver,
     LUSolver,
     CholeskySolver,
+    EigenSolver,
     IterativeLinearSolver,
+    ScipyIterativeSolver,
     CGMatrixSolver,
     BICGMatrixSolver,
     BICGStabMatrixSolver,
@@ -61,3 +70,7 @@ from .linear_optimisation import (
 )
 
 from .linear_bayesian import LinearBayesianInversion, LinearBayesianInference
+
+from .nonlinear_optimisation import (
+    ScipyUnconstrainedOptimiser,
+)
