@@ -38,7 +38,7 @@ import numpy as np
 
 # This block only runs for type checkers, not at runtime
 if TYPE_CHECKING:
-    from .operators import LinearOperator
+    from .linear_operators import LinearOperator
     from .linear_forms import LinearForm
 
 # Define a generic type for vectors in a Hilbert space
@@ -118,6 +118,16 @@ class HilbertSpace(ABC):
             The corresponding vector in the space.
         """
 
+    @abstractmethod
+    def __eq__(self, other: object) -> bool:
+        """
+        Defines equality between two HilbertSpace instances.
+
+        This is an abstract method, requiring all concrete subclasses to
+        implement a meaningful comparison. This ensures that equality is
+        defined by mathematical equivalence rather than object identity.
+        """
+
     # ------------------------------------------------------------------- #
     #            Default implementations that can be overridden           #
     # ------------------------------------------------------------------- #
@@ -209,21 +219,6 @@ class HilbertSpace(ABC):
         """
         return self.from_components(np.random.randn(self.dim))
 
-    def __eq__(self, other: object) -> bool:
-        """
-        Defines equality between Hilbert spaces.
-
-        For dual spaces, equality is determined by the equality of their
-        underlying primal spaces. For non-dual spaces, this is not implemented,
-        requiring concrete subclasses to define a meaningful comparison.
-        """
-        if isinstance(self, DualHilbertSpace):
-            return (
-                isinstance(other, DualHilbertSpace)
-                and self.underlying_space == other.underlying_space
-            )
-        return NotImplemented
-
     # ------------------------------------------------------------------- #
     #                      Final (Non-Overridable) Methods                #
     # ------------------------------------------------------------------- #
@@ -234,7 +229,7 @@ class HilbertSpace(ABC):
         """
         The linear operator mapping R^n component vectors into this space.
         """
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         domain = EuclideanSpace(self.dim)
 
@@ -260,7 +255,7 @@ class HilbertSpace(ABC):
         """
         The linear operator projecting vectors from this space to R^n.
         """
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         codomain = EuclideanSpace(self.dim)
 
@@ -284,7 +279,7 @@ class HilbertSpace(ABC):
     @property
     def riesz(self) -> LinearOperator:
         """The Riesz map (dual to primal) as a `LinearOperator`."""
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         return LinearOperator.self_dual(self.dual, self.from_dual)
 
@@ -292,7 +287,7 @@ class HilbertSpace(ABC):
     @property
     def inverse_riesz(self) -> LinearOperator:
         """The inverse Riesz map (primal to dual) as a `LinearOperator`."""
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         return LinearOperator.self_dual(self, self.to_dual)
 
@@ -415,7 +410,7 @@ class HilbertSpace(ABC):
     @final
     def identity_operator(self) -> LinearOperator:
         """Returns the identity operator `I` on the space."""
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         return LinearOperator(
             self,
@@ -436,7 +431,7 @@ class HilbertSpace(ABC):
         Returns:
             The zero linear operator.
         """
-        from .operators import LinearOperator
+        from .linear_operators import LinearOperator
 
         codomain = self if codomain is None else codomain
         return LinearOperator(
@@ -498,6 +493,25 @@ class DualHilbertSpace(HilbertSpace):
 
         return LinearForm(self._underlying_space, components=c)
 
+    def __eq__(self, other: object) -> bool:
+        """
+        Checks for equality with another dual space.
+
+        Two dual spaces are considered equal if and only if their underlying
+        primal spaces are equal.
+        """
+        if not isinstance(other, DualHilbertSpace):
+            return NotImplemented
+        return self.underlying_space == other.underlying_space
+
+    def is_element(self, x: Any) -> bool:
+        """
+        Checks if an object is a valid element of the dual space.
+        """
+        from .linear_forms import LinearForm
+
+        return isinstance(x, LinearForm) and x.domain == self.underlying_space
+
     @final
     def duality_product(self, xp: LinearForm, x: Vector) -> float:
         """
@@ -572,6 +586,17 @@ class EuclideanSpace(HilbertSpace):
         """Maps a `LinearForm` back to a vector via its components."""
         return self.dual.to_components(xp)
 
+    def __eq__(self, other: object):
+        if not isinstance(other, EuclideanSpace):
+            return NotImplemented
+        return self.dim == other.dim
+
+    def is_element(self, x: Any) -> bool:
+        """
+        Checks if an object is a valid element of the space.
+        """
+        return isinstance(x, np.ndarray) and len(x) == self.dim
+
 
 class MassWeightedHilbertSpace(HilbertSpace):
     """
@@ -651,6 +676,56 @@ class MassWeightedHilbertSpace(HilbertSpace):
         x = self.underlying_space.from_dual(xp)
         return self._inverse_mass_operator(x)
 
+    def __eq__(self, other: object) -> bool:
+        """
+        Checks for equality with another MassWeightedHilbertSpace.
+
+        Two mass-weighted spaces are considered equal if they share an equal
+        underlying space and their mass operators are also equal.
+        """
+        if not isinstance(other, MassWeightedHilbertSpace):
+            return NotImplemented
+
+        return (
+            self.underlying_space == other.underlying_space
+            and (self.mass_operator == other.mass_operator)
+            and (self.inverse_mass_operator == other.inverse_mass_operator)
+        )
+
+    def is_element(self, x: Any) -> bool:
+        """
+        Checks if an object is a valid element of the space.
+        """
+        return self.underlying_space.is_element(x)
+
+    def add(self, x: Vector, y: Vector) -> Vector:
+        """Computes the sum of two vectors. Defaults to `x + y`."""
+        return self.underlying_space.add(x, y)
+
+    def subtract(self, x: Vector, y: Vector) -> Vector:
+        """Computes the difference of two vectors. Defaults to `x - y`."""
+        return self.underlying_space.subtract(x, y)
+
+    def multiply(self, a: float, x: Vector) -> Vector:
+        """Computes scalar multiplication. Defaults to `a * x`."""
+        return self.underlying_space.multiply(a, x)
+
+    def negative(self, x: Vector) -> Vector:
+        """Computes the additive inverse of a vector. Defaults to `-1 * x`."""
+        return self.underlying_space.negative(x)
+
+    def ax(self, a: float, x: Vector) -> None:
+        """Performs in-place scaling `x := a*x`. Defaults to `x *= a`."""
+        self.underlying_space.ax(a, x)
+
+    def axpy(self, a: float, x: Vector, y: Vector) -> None:
+        """Performs in-place operation `y := y + a*x`. Defaults to `y += a*x`."""
+        self.underlying_space.axpy(a, x, y)
+
+    def copy(self, x: Vector) -> Vector:
+        """Returns a deep copy of a vector. Defaults to `x.copy()`."""
+        return self.underlying_space.copy(x)
+
 
 class MassWeightedHilbertModule(MassWeightedHilbertSpace, HilbertModule):
     """
