@@ -24,15 +24,10 @@ from typing import Callable, Optional, Dict, Any
 
 import numpy as np
 from scipy.sparse.linalg import LinearOperator as ScipyLinOp
-from scipy.linalg import (
-    cho_factor,
-    cho_solve,
-    lu_factor,
-    lu_solve,
-)
+from scipy.linalg import cho_factor, cho_solve, lu_factor, lu_solve, eigh
 from scipy.sparse.linalg import gmres, bicgstab, cg, bicg
 
-from .operators import LinearOperator
+from .linear_operators import LinearOperator
 from .hilbert_space import Vector
 
 
@@ -48,6 +43,19 @@ class DirectLinearSolver(LinearSolver):
     factorization.
     """
 
+    def __init__(
+        self, /, *, galerkin: bool = False, parallel: bool = False, n_jobs: int = -1
+    ):
+        """
+        Args:
+            galerkin (bool): If True, the Galerkin matrix representation is used.
+            parallel (bool): If True, parallel computation is used.
+            n_jobs (int): Number of parallel jobs.
+        """
+        self._galerkin: bool = galerkin
+        self._parallel: bool = parallel
+        self._n_jobs: int = n_jobs
+
 
 class LUSolver(DirectLinearSolver):
     """
@@ -55,12 +63,16 @@ class LUSolver(DirectLinearSolver):
     dense matrix representation.
     """
 
-    def __init__(self, /, *, galerkin: bool = False) -> None:
+    def __init__(
+        self, /, *, galerkin: bool = False, parallel: bool = False, n_jobs: int = -1
+    ) -> None:
         """
         Args:
             galerkin (bool): If True, the Galerkin matrix representation is used.
+            parallel (bool): If True, parallel computation is used.
+            n_jobs (int): Number of parallel jobs.
         """
-        self._galerkin: bool = galerkin
+        super().__init__(galerkin=galerkin, parallel=parallel, n_jobs=n_jobs)
 
     def __call__(self, operator: LinearOperator) -> LinearOperator:
         """
@@ -74,7 +86,12 @@ class LUSolver(DirectLinearSolver):
         """
         assert operator.is_square
 
-        matrix = operator.matrix(dense=True, galerkin=self._galerkin)
+        matrix = operator.matrix(
+            dense=True,
+            galerkin=self._galerkin,
+            parallel=self._parallel,
+            n_jobs=self._n_jobs,
+        )
         factor = lu_factor(matrix, overwrite_a=True)
 
         def matvec(cy: np.ndarray) -> np.ndarray:
@@ -102,12 +119,16 @@ class CholeskySolver(DirectLinearSolver):
     representation is positive-definite.
     """
 
-    def __init__(self, /, *, galerkin: bool = False) -> None:
+    def __init__(
+        self, /, *, galerkin: bool = False, parallel: bool = False, n_jobs: int = -1
+    ) -> None:
         """
         Args:
             galerkin (bool): If True, the Galerkin matrix representation is used.
+            parallel (bool): If True, parallel computation is used.
+            n_jobs (int): Number of parallel jobs.
         """
-        self._galerkin: bool = galerkin
+        super().__init__(galerkin=galerkin, parallel=parallel, n_jobs=n_jobs)
 
     def __call__(self, operator: LinearOperator) -> LinearOperator:
         """
@@ -121,7 +142,12 @@ class CholeskySolver(DirectLinearSolver):
         """
         assert operator.is_automorphism
 
-        matrix = operator.matrix(dense=True, galerkin=self._galerkin)
+        matrix = operator.matrix(
+            dense=True,
+            galerkin=self._galerkin,
+            parallel=self._parallel,
+            n_jobs=self._n_jobs,
+        )
         factor = cho_factor(matrix, overwrite_a=False)
 
         def matvec(cy: np.ndarray) -> np.ndarray:
@@ -136,6 +162,78 @@ class CholeskySolver(DirectLinearSolver):
         )
 
 
+class EigenSolver(DirectLinearSolver):
+    """
+    A direct linear solver based on the eigendecomposition of a symmetric operator.
+
+    This solver is robust for symmetric operators that may be singular or
+    numerically ill-conditioned. In such cases, it computes a pseudo-inverse by
+    regularizing the eigenvalues, treating those close to zero (relative to the largest
+    eigenvalue) as exactly zero.
+    """
+
+    def __init__(
+        self,
+        /,
+        *,
+        galerkin: bool = False,
+        parallel: bool = False,
+        n_jobs: int = -1,
+        rtol: float = 1e-12,
+    ) -> None:
+        """
+        Args:
+            galerkin (bool): If True, the Galerkin matrix representation is used.
+            parallel (bool): If True, parallel computation is used.
+            n_jobs (int): Number of parallel jobs.
+            rtol (float): Relative tolerance for treating eigenvalues as zero.
+                An eigenvalue `s` is treated as zero if
+                `abs(s) < rtol * max(abs(eigenvalues))`.
+        """
+        super().__init__(galerkin=galerkin, parallel=parallel, n_jobs=n_jobs)
+        self._rtol = rtol
+
+    def __call__(self, operator: LinearOperator) -> LinearOperator:
+        """
+        Computes the pseudo-inverse of a self-adjoint LinearOperator.
+        """
+        assert operator.is_automorphism
+
+        matrix = operator.matrix(
+            dense=True,
+            galerkin=self._galerkin,
+            parallel=self._parallel,
+            n_jobs=self._n_jobs,
+        )
+
+        eigenvalues, eigenvectors = eigh(matrix)
+
+        max_abs_eigenvalue = np.max(np.abs(eigenvalues))
+        if max_abs_eigenvalue > 0:
+            threshold = self._rtol * max_abs_eigenvalue
+        else:
+            threshold = 0
+
+        inv_eigenvalues = np.where(
+            np.abs(eigenvalues) > threshold,
+            np.reciprocal(eigenvalues),
+            0.0,
+        )
+
+        def matvec(cy: np.ndarray) -> np.ndarray:
+            z = eigenvectors.T @ cy
+            w = inv_eigenvalues * z
+            return eigenvectors @ w
+
+        inverse_matrix = ScipyLinOp(
+            (operator.domain.dim, operator.codomain.dim), matvec=matvec, rmatvec=matvec
+        )
+
+        return LinearOperator.from_matrix(
+            operator.domain, operator.domain, inverse_matrix, galerkin=self._galerkin
+        )
+
+
 class IterativeLinearSolver(LinearSolver):
     """
     An abstract base class for iterative linear solvers.
@@ -172,7 +270,10 @@ class IterativeLinearSolver(LinearSolver):
         """
         Solves the adjoint linear system A*y = x for y.
         """
-        return self.solve_linear_system(operator.adjoint, preconditioner.adjoint, x, y0)
+        adjoint_preconditioner = (
+            None if preconditioner is None else preconditioner.adjoint
+        )
+        return self.solve_linear_system(operator.adjoint, adjoint_preconditioner, x, y0)
 
     def __call__(
         self,
