@@ -24,7 +24,7 @@ Key Classes
 
 from __future__ import annotations
 
-from typing import Callable, Tuple, Optional
+from typing import Callable, Tuple, Optional, Any
 import matplotlib.pyplot as plt
 import numpy as np
 from scipy.fft import rfft, irfft
@@ -38,7 +38,7 @@ from pygeoinf.hilbert_space import (
     HilbertModule,
     MassWeightedHilbertModule,
 )
-from pygeoinf.operators import LinearOperator
+from pygeoinf.linear_operators import LinearOperator
 from pygeoinf.linear_forms import LinearForm
 from .symmetric_space import (
     AbstractInvariantLebesgueSpace,
@@ -336,18 +336,31 @@ class Lebesgue(CircleHelper, HilbertModule, AbstractInvariantLebesgueSpace):
 
         return self.kmax == other.kmax and self.radius == other.radius
 
-    def invariant_automorphism(self, f: Callable[[float], float]):
+    def is_element(self, u: Any) -> bool:
+        """
+        Checks if an object is a valid element of the space.
+        """
+        if not isinstance(u, np.ndarray):
+            return False
+        if not u.shape == (self.dim,):
+            return False
+        return True
+
+    def invariant_automorphism_from_index_function(self, g: Callable[[int], float]):
         """
         Implements an invariant automorphism of the form f(Δ) using Fourier
         expansions on a circle.
 
+        For this method, the function f is given implicitly in terms of a
+        function, g, of the eigenvalue indices for the space. Letting k(λ) be
+        the index for eigenvalue λ, we then have f(λ) = g(k(λ)).
+
         Args:
-            f: A real-valued function that is well-defined on the spectrum
-               of the Laplacian, Δ.
+            g: A real-valued function of the eigenvalue index.
         """
 
         values = np.fromiter(
-            (f(self.laplacian_eigenvalue(k)) for k in range(self.kmax + 1)),
+            (g(k) for k in range(self.kmax + 1)),
             dtype=float,
         )
         matrix = diags([values], [0])
