@@ -38,7 +38,7 @@ from pygeoinf.hilbert_space import (
     HilbertModule,
     MassWeightedHilbertModule,
 )
-from pygeoinf.operators import LinearOperator
+from pygeoinf.linear_operators import LinearOperator
 from pygeoinf.linear_forms import LinearForm
 from .symmetric_space import (
     AbstractInvariantLebesgueSpace,
@@ -160,6 +160,17 @@ class SphereHelper:
         l = k[0]
         return l * (l + 1) / self.radius**2
 
+    def degree_from_laplacian_eigenvalue(self, eig: float) -> float:
+        """
+        Returns the degree corresponding to a given eigenvalue.
+
+        Note that the value is returned as a float
+
+        Args:
+            eig: The eigenvalue.
+        """
+        return np.sqrt(self.radius**2 * eig + 0.25)
+
     def trace_of_invariant_automorphism(self, f: Callable[[float], float]) -> float:
         """
         Returns the trace of the automorphism of the form f(Δ) with f a function
@@ -425,6 +436,32 @@ class Lebesgue(SphereHelper, HilbertModule, AbstractInvariantLebesgueSpace):
         """
         return x1 * x2
 
+    def __eq__(self, other: object) -> bool:
+        """
+        Checks for mathematical equality with another Sobolev space on a sphere.
+
+        Two spaces are considered equal if they are of the same type and have
+        the same defining parameters (kmax, order, scale, and radius).
+        """
+        if not isinstance(other, Lebesgue):
+            return NotImplemented
+
+        return self.lmax == other.lmax and self.radius == other.radius
+
+    def is_element(self, x: Any) -> bool:
+        """
+        Checks if an object is a valid element of the space.
+        """
+        if not isinstance(x, sh.SHGrid):
+            return False
+        if not x.lmax == self.lmax:
+            return False
+        if not x.grid == self._grid_name():
+            return False
+        if not x.extend == self.extend:
+            return False
+        return True
+
     def eigenfunction_norms(self) -> np.ndarray:
         """Returns a list of the norms of the eigenfunctions."""
         return np.fromiter(
@@ -432,10 +469,10 @@ class Lebesgue(SphereHelper, HilbertModule, AbstractInvariantLebesgueSpace):
             dtype=float,
         )
 
-    def invariant_automorphism(self, f: Callable[[float], float]) -> LinearOperator:
-        values = self._degree_dependent_scaling_values(
-            lambda l: f(self.laplacian_eigenvalue((l, 0)))
-        )
+    def invariant_automorphism_from_index_function(
+        self, g: Callable[[(int, int)], float]
+    ) -> LinearOperator:
+        values = self._degree_dependent_scaling_values(lambda l: g((l, 0)))
         matrix = diags([values], [0])
 
         def mapping(u):
@@ -445,9 +482,15 @@ class Lebesgue(SphereHelper, HilbertModule, AbstractInvariantLebesgueSpace):
 
         return LinearOperator.self_adjoint(self, mapping)
 
-    # ================================================================ #
-    #                         Private methods                          #
-    # ================================================================ #
+    def __str__(self) -> str:
+        """Returns a human-readable string summary of the space."""
+        return (
+            f"Lebesgue space on sphere:\n"
+            f"lmax={self.lmax}\n"
+            f"radius={self.radius}\n"
+            f"grid={self.grid}\n"
+            f"extend={self.extend}"
+        )
 
 
 class Sobolev(SphereHelper, MassWeightedHilbertModule, AbstractInvariantSobolevSpace):
@@ -598,3 +641,15 @@ class Sobolev(SphereHelper, MassWeightedHilbertModule, AbstractInvariantSobolevS
 
         c = self._coefficient_to_component(ulm)
         return self.dual.from_components(c)
+
+    def __str__(self) -> str:
+        """Returns a human-readable string summary of the space."""
+        return (
+            f"Lebesgue space on sphere:\n"
+            f"lmax={self.lmax}\n"
+            f"order={self.order}\n"
+            f"scale={self.scale}\n"
+            f"radius={self.radius}\n"
+            f"grid={self.grid}\n"
+            f"extend={self.extend}"
+        )
